import wx
import  wx.stc  as  stc
from skin import PythonSkin
import types
import keyword
import os
import os.path

from customSTC import *

#-------------------------- some commands--------------------------------------------
def PythonComment_CMD(target):
        sel = target.GetSelection()
        start = target.LineFromPosition(sel[0])
        end = target.LineFromPosition(sel[1])
        if end > start and target.GetColumn(sel[1]) == 0:
            end = end - 1
        target.BeginUndoAction()
        try:
            for lineNumber in range(start, end + 1):
                target.ToggleCommentLine(lineNumber)
            target.SetCurrentPos(target.PositionFromLine(start))
            target.SetAnchor(target.GetLineEndPosition(end))
        finally:
            target.EndUndoAction()

def SaveFile_CMD(target):
    if target.filename:
        target.SaveFile(target.filename)
    else:
        target.GetParent().SaveCurrentFile()
def Revert_CMD(target):
    if target.filename:
        target.LoadFile(target.filename)

def ResetZoom_CMD(target):
    target.SetZoom(0)

#----------------------------------------------------------------------
python_suppliment_commands = {
'CTRL+Q':PythonComment_CMD,
'CTRL+S':SaveFile_CMD,
'CTRL+R':Revert_CMD,
'CTRL+DIVIDE':ResetZoom_CMD,
}
#----------------------------------------------------------------------
class PyMiniEd(CustomSTC):
    def __init__(self, parent, filename = None):
        CustomSTC.__init__(self, parent)
        self.SetLexer(stc.STC_LEX_PYTHON)
        self.SetKeyWords(0, " ".join(keyword.kwlist))
        PythonSkin(self)        
        self.comment_particle = "# "
        self.command_map.update(python_suppliment_commands)
        self.dirty = False
        if filename:
            self.LoadFile(filename)
            self.filename = filename
            self.shortname = os.path.basename(filename)
        else:
            self.filename = None
            self.shortname = "Untitled"
            
    def ToggleCommentLine(self, lineNumber):
        firstChar = self.GetLineIndentPosition(lineNumber)
        mark =  self.GetTextRange(firstChar,min(self.GetLength(), firstChar+3))
        if mark.startswith("## "):
            self.SetSelection(firstChar, firstChar+3)        
        elif mark.startswith("# "):
            self.SetSelection(firstChar, firstChar+2)
        elif mark.startswith("#"):
            self.SetSelection(firstChar, firstChar+1)
        else:
            self.InsertText(firstChar, self.comment_particle)
            return
        self.ReplaceSelection("")

#----------------------------------------------------------------------
class EdNotebook(wx.Notebook):
    def __init__(self, parent):
        wx.Notebook.__init__(self, parent)
        self.command_map = {
            "CTRL+N":EdNotebook.LoadFile,
            "CTRL+O":EdNotebook.Open,
            "CTRL+W":lambda s:wx.CallAfter(EdNotebook.CloseCurrent, s),
            # "CTRL+W":EdNotebook.CloseCurrent, #this one segfaults on windows
            }
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyPressed)
        self.LoadFile(None)
        
    def OnKeyPressed(self, evt):
        key_name = decode_key_event(evt)
        self.OnCommandIssued(CommandIssuedEvent(self.GetId(), key_name))
            
    def OnCommandIssued(self, evt):
        if evt.cmd in self.command_map:
            self.command_map[evt.cmd](self)
        else:
            self.GetEventHandler().ProcessEvent(CommandIssuedEvent(self.GetId(), evt.cmd))
        
    def SetDirty(self, evt):
        page = self.GetPage(self.GetSelection())
        page.dirty = True
        self.SetPageText(self.GetSelection(), page.shortname+" *")
        
    def SetClean(self, evt):
        page = self.GetPage(self.GetSelection())
        page.dirty = False
        self.SetPageText(self.GetSelection(), page.shortname)
        
    def LoadFile(self, file=None):
        page = PyMiniEd(self, file)
        self.AddPage(page, page.shortname)
        page.Bind(wx.stc.EVT_STC_SAVEPOINTLEFT, self.SetDirty)
        page.Bind(wx.stc.EVT_STC_SAVEPOINTREACHED, self.SetClean)
        page.Bind(EVT_COMMAND_ISSUED, self.OnCommandIssued)
        self.SetSelection(self.GetPageCount()-1) #last page
        page.SetFocus()
        
    def SaveCurrentFile(self):
        dlg = wx.FileDialog(
            self, message="Save file as ...", defaultDir=os.getcwd(), 
            defaultFile="Untitled.py", wildcard="Python source (*.py)|*.py", style=wx.SAVE
            )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            page = self.GetPage(self.GetSelection())
            page.filename = path
            page.shortname = os.path.basename(path)
            page.SaveFile(path)
            
    def Open(self):
        dlg = wx.FileDialog(
            self, message="Save file as ...", defaultDir=os.getcwd(), 
            defaultFile="Untitled.py", wildcard="Python source (*.py)|*.py", style=wx.OPEN
            )
        if dlg.ShowModal() == wx.ID_OK:
            self.LoadFile(dlg.GetPath())
            
    def CloseCurrent(self):
        if self.GetPageCount():
            sel = self.GetSelection()
            page = self.GetPage(sel)
            if page.dirty:
                dlg = wx.MessageDialog(self, 'There are unsaved changes.\nClose without save?',
                       'Unsaved Changes!',wx.YES_NO | wx.ICON_QUESTION | wx.NO_DEFAULT)
                if dlg.ShowModal() == wx.ID_YES:
                    self.DeletePage(sel)
                dlg.Destroy()
            else:
                self.DeletePage(sel)
        if not self.GetPageCount():
            self.LoadFile(None)
    def Close(self):
        for i in range(0, self.GetPageCount()):
            wx.CallAfter(self.CloseCurrent)
        # returns safe to close if there is only one empty page 
        return self.GetPageCount() == 1 and not self.GetPage(self.GetSelection()).GetText()
        
class MiniEdFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, title="MiniEd")
        self.nb = EdNotebook(self)
        self.SetSize((800,600))
        self.CenterOnScreen()
        self.command_map = {
            "CTRL+F4":MiniEdFrame.Close,
        }
        self.nb.Bind(EVT_COMMAND_ISSUED, self.OnCommandIssued)
        self.Show()
        
    def Close(self):
        if self.nb.Close(): #if the notebook is safe to close
            wx.Frame.Close(self)
            
    def OnCommandIssued(self, evt):
        if evt.cmd in self.command_map:
            self.command_map[evt.cmd](self)
            
if __name__ == "__main__":
    app = wx.App(0)
    frame = MiniEdFrame()
    app.MainLoop()
